#include <OneWire.h>
#include <SoftwareSerial.h>
#include <EEPROM.h>
#include <MsTimer2.h>
#include <RTClib.h>




#define ON 1                          // Включено
#define OFF 0                         // Выключено  
#define PHONE_COUNT 2                 // Количество номеров
#define SMS true                      // Отправлять СМС
#define INT_INTERVAL 15000
#define BALANCE_LEN 22                // Длина ответа на запрос баланса
#define BATTERY_VOLTAGE 4.2
#define BATTERY_LOW_VOLTAGE 3.3
#define BATTERY_NORMAL_VOLTAGE 3.8
#define GAS_VALUE 800
#define ALARM_TEMP 60.0
//#define TEMPERATURE_PRECISION 9       // Разрешение температуры
#define POWER_MODE  0       // внешнее питание датчика.

// --------- Раскладка EEPROM -----------------------------------------------------

#define EEPROM_IS_CONFIG 0            // Сброс мастер номера
#define EEPROM_CONFIG_ADDRESS 10      // Конфигурация Auto
#define EEPROM_SETTINGS_ADDRESS 100   // Настройки
#define PHONE1 300                   // Мастер номер
#define PHONE2 350                   // Второй номер
#define BAL_NUM 380                  // Номер запроса баланса

// ----------- Порты --------------------------------------------------------------
             
#define OPEN_DET 2                  // концевик двери
#define WINDOW_DET 3                // Датчик открытия окна
#define MAIN_POWER 4                // Вход контроля наличия основного питания () 
#define MOVE 5                      // Датчик движения
#define ALARM_LED 6                 // Индикатор охраны и состояния датчика
#define INDICATOR 7                 // Индикатор работы   INDICATOR ------|<|---|10k|---> +VCC
#define M_RX 8                      // К UART модема (TX)  
#define M_TX 9                      // К UART модема (RX)
#define DS18B20 10                  // Датчик температуры
#define OUT_PIN 11                  // Выход на блок реле
#define OUT_PIN_2 12                // Выход на блок реле
#define CONFIG 13                   // Кнопка записи мастера
#define GAS A0                      // Датчик газа и дыма
#define BATTERY A6                  // Вход контроля напряжения батареи (14)     

// -------------------------------- Дни недели -------------------------------------
#define SUNDAY 0                      // Воскресенье
#define MONDAY 1                      // Понедельник
#define TUESDAY 2                     // Вторник
#define WEDNESDAY 3                   // Среда
#define THURSDAY 4                    // Четверг
#define FRIDAY 5                      // Пятница
#define SATURDAY 6                    // Суббота

// -------------------------------
void(* resetFunc) (void) = 0;                // Функция сброса стека вызова

SoftwareSerial mGSM(M_RX, M_TX);             // Цепляем софт UART для модема на пины M_RX, M_TX
RTC_DS1307 rtc;                              // объявляем экземпляр RTC
OneWire sensors(DS18B20);


// ------------------ Структура хранения настроек -----------------------------------------------
struct settings {
  int phone_count;
  int bal_len;
  long interval;
  boolean temp_alarm;
  boolean open_alarm;
  boolean gas_alarm;
  boolean window_alarm;
  boolean move_alarm;
  boolean guard;
  int gas_alarm_value;
} dev_config;

// ------------------------------- Структура хранения конфигурации ------------------------------

const char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

boolean MAIN_POWER_FLAG = true;  // Флаг пропадания основного питания
boolean LOW_BATTERY_FLAG = true; // Флаг низкого заряда батареи
boolean OPEN_FLAG = true;   // Флаг авария по температуре
boolean OPEN_ALARM = true;
boolean WINDOW_ALARM = true;
boolean WINDOW_FLAG = true;
boolean TEMP_ALARM = true;
boolean TEMP_FLAG = true;
boolean MOVE_ALARM = true;
boolean MOVE_FLAG = true;
boolean GAS_FLAG = true;
boolean GAS_ALARM = true;
boolean FIRST_START = false;
boolean IS_RTC = false;
boolean out_state[2];

unsigned long last_millis = 0;
unsigned long stop_time[2];


float voltage_f;                            // Напряжение АКБ
float now_temp;                 // Текущая температура

byte ch = 0;
byte bufData[9];

String val = "", number;                    // Буфер UART модема
String RingPhone = "";                      // Номер абонента
String Ops="No network";                    // Хранит название сети
String AllowPhones[PHONE_COUNT];

int CountPhone;                             // Количество телефонов
int voltage_i;                              // Данные с АЦП напряжения АКБ
int GAS_ALARM_VALUE=1023;                   // Данные с АЦП напряжение с датчика MQ-2
int pin_out[2]={ OUT_PIN, OUT_PIN_2 };

void setup() {
  MsTimer2::set(5000, flash);                // 5000ms период мырга
                                            // Стартуем таймер
  analogReference(EXTERNAL);                // Опорное внешнее будет 2,5в с TL431 (Aref)
  Serial.begin(9600);  
  mGSM.begin(9600);                         // Стартуем аппаратный UART
  
  
  if (! rtc.begin()) { IS_RTC=false; } else { IS_RTC=true; }
  if (! rtc.isrunning()) { rtc.adjust(DateTime(F(__DATE__), F(__TIME__))); 
  //resetFunc();
  } 
  
  pinMode(INDICATOR, OUTPUT);               // Пин индикатора работы (выход)
  pinMode(ALARM_LED, OUTPUT);               // Пин индикатора режимов охраны
  pinMode(MAIN_POWER,INPUT);
  pinMode(CONFIG,INPUT_PULLUP);
  pinMode(OPEN_DET,INPUT);
  pinMode(WINDOW_DET, INPUT);
  pinMode(OUT_PIN, OUTPUT);
  pinMode(OUT_PIN_2, OUTPUT);
  
  //reset_config();  
  
  load_settings();                          // Загружаем настройки
  initModem();                              // Инициализируем модем
  MsTimer2::start(); 
  
}
void loop() {
  
  if (digitalRead(CONFIG)==LOW) {
    _delay_ms(1000);
    EEPROM.write(EEPROM_IS_CONFIG, 0);
    if (dev_config.phone_count>0) { sms("Make ring from new master",String(AllowPhones[0]));}
   }

  if (last_millis > millis())
    last_millis = 0;                        //  сброс счетчика при переполнении.

  if (millis() - last_millis > INT_INTERVAL) {  // Выполнение операций по интервалу
     last_millis = millis();
     
    for (int i=0;i<2;i++){
      if (stop_time[i]>millis()) {set_out(i,true);} else {set_out(i,false);
      }
    }
     
  sensors.reset();  // 
  sensors.write(0xCC, POWER_MODE); // 
  sensors.write(0x44, POWER_MODE); // 
  _delay_ms(1000);  // 
  sensors.reset();  // 
  sensors.write(0xCC, POWER_MODE); // 
  sensors.write(0xBE, POWER_MODE); // 
  sensors.read_bytes(bufData, 9);  // 

  if ( OneWire::crc8(bufData, 8) == bufData[8] ) {  // 
        now_temp=  (float)((int)bufData[0] | (((int)bufData[1]) << 8)) * 0.0625 + 0.03125; }
        else now_temp=0.0;
    
  }

  digitalWrite(ALARM_LED, dev_config.guard); // Отобразить состояние охраны.
  
  detect_power_off();                       // Проверяем наличие питания
  detect_low_power();                       // Проверяем напряжение батареи
  detect_temp();
  if(dev_config.guard) {
  detect_open();
  detect_gas();
  detect_window();
  }
   
  if (mGSM.available()) {                   // Считываем данные с UART
    while (mGSM.available()) {              //
      ch = mGSM.read();                     //
      val += char(ch);                      //
      _delay_ms(10);                        //
    }
    Serial.println(val);                    // Дублируем в аппаратный UART для терминала

    if (val.indexOf(F("RING")) > -1) {      // Если поступил вызов
      _delay_ms(100);
      if (CheckPhone() == 1) {              // Проверяем телефон и если это наш
        mGSM.println(F("ATH0"));            // Бросаем трубку
        MasterRing();                       // Событие по звонку
        val = "";                           // Обнуляем буфер
      } else {
        mGSM.println(F("ATH0"));            // Если не наш - просто сбросим
      }
    } else if (val.indexOf(F("+CMT:")) > -1) { // Если получен месседж
      if (CheckPhone() == 1) {              // Проверка телефона
        MasterSms();                        // Обработка СМС
        val="";
      }
    } else if (val.indexOf(F("+CUSD:")) > -1) { // Если получен ответ USSD
      Serial.println(RingPhone);              // Кто отправлял запрос
      Serial.println(val);                    // Полученный ответ
      if (val.indexOf(F("\"")) > -1) {          // Обработка UDP
        String str;
        str = val.substring(String(val.indexOf(",\"")).toInt() + 2, String(val.indexOf(",\"")).toInt() + BALANCE_LEN + 6);

        Serial.println(str);
        sms(str, String(RingPhone)); // Отправка ответа на запрос
        _delay_ms(1000);
      }
      mGSM.println(F("AT+CUSD=0"));
      _delay_ms(200);
    } else if (val.indexOf(F("+PBREADY")) > -1) { // Модем ответил, что он стартанул
      initModem();                         // Инициализируем модем
    } else if (val.indexOf(F("+COPS:")) > -1) { // Получаем Опсоса
      Ops=val.substring(val.indexOf(",\"") + 2, val.length()-9);
      val = ""; 
      Serial.print("Network:");
      Serial.println(Ops);
    }
    //  Serial.println(val);
    val = "";                              // Очистим буфер UART
  }

  // ------------------- зацепим консоль -----------------------------------
  if (Serial.available()) {                //если в мониторе порта ввели что-то
    while (Serial.available()) {           //сохраняем строку в переменную val
      ch = Serial.read();
      val += char(ch);
      _delay_ms(10);
    }
    consolePrint();                        // Обработка команд
    val = "";                              // Очищаем буфер
  }
}
// ---------------- Включение и выключение управляющих выходов -------------------
void set_out(int out, boolean state) {
  String msg=F("Out[");
  msg +=String(out+1);
  msg +=F("]:");
  if(!TEMP_FLAG){
  if (state) {
    digitalWrite(pin_out[out], HIGH);  // Включить реле
    msg +=F("ON");
    if (!out_state[out]) sms(msg,AllowPhones[0]);
    out_state[out] = true;                               // Статус -> вкл.
  
  } else {
    msg +=F("OFF");
    digitalWrite(pin_out[out], LOW); // Выключить реле
    if (out_state[out]) sms(msg,AllowPhones[0]);
    out_state[out] = false;                              // Статус -> выкл.
  }
  _delay_ms(50);
  Serial.println(msg);
  } else {
  digitalWrite(pin_out[0],false);
  digitalWrite(pin_out[1],false);
  out_state[0] = false;
  out_state[1] = false;
  }
}
// ------------------------------------------------ Функция по прерыванию --------------------------------------------------
void flash() {
  static boolean output = HIGH;
  if (OPEN_FLAG || GAS_FLAG || MOVE_FLAG || WINDOW_FLAG) {
    digitalWrite(ALARM_LED, !output);
  } else {
    digitalWrite(ALARM_LED, LOW);
  }
  
  digitalWrite(INDICATOR, output);
  
  output = !output;
  
}
// ------------------------------------------------ Обработка звонков ------------------------------------------------------
void MasterRing() {    
  String msg=F("Guard is:");
  dev_config.guard = !dev_config.guard; // Снятие и постановка в охрану.
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  val=""; // Очищаем буфер модема.
  if(dev_config.guard) msg +=F("ON."); else msg +=F("OFF.");
  sms(msg,RingPhone);
  }
//------------------------------------------------- Определить состояние питания батареи -----------------------------------
void detect_low_power() {
  int j;
  voltage_i = analogRead(BATTERY);                                        // Получаем значение АЦП
  voltage_f = (BATTERY_VOLTAGE / 1024) * voltage_i;                       // Вычисляем значение напряжения
  
  // Проверяем питание, устанавливаем флаг
  if (digitalRead(MAIN_POWER) == LOW) {                                   // Если основного питания нет то мониторим АКБ

    if (voltage_f < BATTERY_LOW_VOLTAGE && LOW_BATTERY_FLAG == false) {   // Если напряжение ниже порогового
      LOW_BATTERY_FLAG = true;                                            // Установка флага
      for (j = 0; j < CountPhone; j++) {                                   // Сообщаем СМСкой
        sms(F("Low Battery! "), AllowPhones[j]);                          //
        _delay_ms(2000);
        }                              
      Serial.println(voltage_f);                                          //
    }
    if (voltage_f >= BATTERY_NORMAL_VOLTAGE && LOW_BATTERY_FLAG == true) {// Если напряжение выше порога, то
      LOW_BATTERY_FLAG = false;                                           // Снимаем флаг
    }
  }
}
// ------------------------------------------------ Обнаружение отключения и включения питания -----------------------------
void detect_power_off() {
  String msg = F("Main Power ") ;
  if (digitalRead(MAIN_POWER) == LOW && MAIN_POWER_FLAG == false)       // Если пропало питание от сети
  { msg += F("OFF");
    Serial.println(msg);                            // Сообщаем СМСкой
    for (int j = 0; j < CountPhone; j++) {                                // на все номера
      sms(msg, String(AllowPhones[j]));   //
    _delay_ms(2000);
    }
    MAIN_POWER_FLAG = true;                                             // Устанавливаем флаг

  }
  if (digitalRead(MAIN_POWER) ==HIGH && MAIN_POWER_FLAG == true)       // Если основное питание появилось
  { msg += F("ON");
    Serial.println(msg);
    for (int j = 0; j < CountPhone; j++) {                             // Сообщаем СМСкой
      sms(msg, String(AllowPhones[j]));
      _delay_ms(2000);
      }
    MAIN_POWER_FLAG = false;                                            // Снимаем флаг
  }
}
// ------------------------------------------------ Сброс программный ------------------------------------------------------
void reboot(String phone) {
  String str = F("reboot by: ");
  str += phone;                             // Кто отправил команду
  for (int j = 0; j < CountPhone; j++) {
    sms(str, String(AllowPhones[j])); 
    _delay_ms(2000);// Отправка СМС о перезапуске
  }
  resetFunc();                              // Перезапуск
}
// ------------------------------------------------ Инициализация модема ---------------------------------------------------
void initModem() {
  Serial.println(F("Wait 5 s init modem"));
  _delay_ms(5000);
  Serial.println(F("Init modem"));
  mGSM.println(F("ATI"));                   // Моделька модема
  _delay_ms(200);
  mGSM.println(F("AT+CLIP=1"));             // Определитель номера
  _delay_ms(200);
  mGSM.println(F("AT+CMGF=1"));             // Режим текстовый
  _delay_ms(200);
  mGSM.println(F("AT+CSCS=\"GSM\""));       // Кодировка GSM
  _delay_ms(200);
  mGSM.println(F("AT+CNMI=2,2"));           //отображение смс в терминале сразу после приема (без этого сообщения молча падают в память) "AT+CNMI=2,2"
  _delay_ms(200);
  clearsms("", false);   
  _delay_ms(500);// Очистка памяти СМС
  mGSM.println(F("AT+COPS=0,0"));
  _delay_ms(500);
  mGSM.println(F("AT+COPS?"));
  _delay_ms(500);
  
}
// ------------------------------------------------ Проверка телефона ------------------------------------------------------
int CheckPhone() {
  if (EEPROM.read(EEPROM_IS_CONFIG) == 0) {                              // Если мастер номер не сконфигурирован
    String buf = String("+") + val.substring(val.indexOf(": \"") + 3, val.indexOf("\",")); //Получаем номер телефона из строки
    char cbuf[20];
    buf.toCharArray(cbuf, 20);
    EPSW(PHONE1, cbuf);                                              // Пишем в память
    AllowPhones[0] = String((char*)cbuf);                            // Обновляем текущий массив номеров
    dev_config.phone_count = 1;                                          // Записываем количество

    EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);                     // Сохраняем данные и настройки в EEPROM
    Serial.println("Master :" + EPSR(PHONE1));
    EEPROM.write(EEPROM_IS_CONFIG, 1);                                   // Запрещаем запись мастер номера по первому звонку
    sms("It is master now!", String(AllowPhones[0])); 
  }
  
  for (int i = 0; i < dev_config.phone_count; i++) {                                 // Если номер наш
    Serial.println(AllowPhones[i]);
    Serial.println(AllowPhones[i].substring(1,AllowPhones[i].length()));
    Serial.println(val.indexOf(AllowPhones[i].substring(1,AllowPhones[i].length()-1)));
    Serial.println(val);
    String tmp=AllowPhones[i].substring(1,AllowPhones[i].length());
    if (val.indexOf(tmp) > -1) {
      
      RingPhone = AllowPhones[i];                                        // Запоминаем кто звонил
      Serial.print(F("Sobitie:  "));
      Serial.println(RingPhone);
      return 1;                                                          // Возвращаем что номер наш
    }
  }
  return 0;                                                              // Если номер не наш, то вернем 0
}
// ------------------------------------------------ Запрос баланса ---------------------------------------------------------
void balance() {
  String balance_num;
  balance_num = EPSR(BAL_NUM);
  mGSM.println(F("AT+CUSD=1")); // Режим запроса с ответом
  _delay_ms(500);
  mGSM.print(F("ATD"));         // Запрос
  balance_num.trim();
  mGSM.println(balance_num);
  _delay_ms(200);
}
// ------------------------------------------------ Чтение команд с консоли ------------------------------------------------
void consolePrint() {
  val.toLowerCase();                                                  // Переводим строку в нижний регистр

  if (val.indexOf(F("info")) > -1) {                           // если обнаружено кодовое слово info
    info("", false);                                                  // Информация о состоянии
  } else if ((val.indexOf(F("reboot")) > -1)) {                       // Команда на перезапуск
    reboot(F("Direct reboot"));
  } else if ((val.indexOf(F("reconfig")) > -1)) {                     // Сброс конфигурации и детектирование датчиков
    reset_config();
  } else if ((val.indexOf(F("balnum:")) > -1)) {                   // Изменение номера запроса баланса setbalnum:#105#;22
    set_bal_num(val, false);
  } else if ((val.indexOf(F("addphone:")) > -1)) {                    // Добавить/изменить телефон в памяти addphone:+79876543210ms
    add_phone(false);
  } else if ((val.indexOf(F("newmaster")) > -1)) {                    // Запись нового мастер телефона по звонку
    EEPROM.write(EEPROM_IS_CONFIG, 0);
  } else if ((val.indexOf(F("clearsms")) > -1)) {                     // Удалить все СМС
    _delay_ms(1000);
    clearsms(RingPhone, false);
    _delay_ms(1000);
  } else if ((val.indexOf(F("int:")) > -1)) {                 // Установить интервал обновления данных
    set_interval(false);
  } else if ((val.indexOf(F("phones")) > -1)) {                       // Вывести список телефонов
    get_phones();
  } else if ((val.indexOf(F("sendat:")) > -1)) {                      // Отправить команду модему
    mGSM.println(val.substring(val.indexOf(F("at:")) + 3));
  } else if (val.indexOf(F("gas:on")) > -1) {
    gas_state(true,RingPhone,false);
  } else if (val.indexOf(F("gas:off")) > -1) {
    gas_state(false,RingPhone,false);
  } else if (val.indexOf(F("gaslevel:")) > -1) { 
    gas_level(val,RingPhone,false);
  } else if (val.indexOf(F("door:on")) > -1) {
    open_state(true,RingPhone,true);
  } else if (val.indexOf(F("door:off")) > -1) {
    open_state(false,RingPhone,true);
  } else if (val.indexOf(F("window:on")) > -1) {
    window_state(true,RingPhone,false);
  } else if (val.indexOf(F("window:off")) > -1) {
    window_state(false,RingPhone,false);
  } else if (val.indexOf(F("move:on")) > -1) {
    move_state(true,RingPhone,true);
  } else if (val.indexOf(F("move:off")) > -1) {
    move_state(false,RingPhone,true);
  } else if ((val.indexOf(F("bytime[")) > -1)) {                   // Включение нагрузки на время в часах.
    by_time(val, false);
  } else if ((val.indexOf(F("stoptime:")) > -1)) {                   // Отключение нагрузки.
    set_stop_time(val, false);
  }

}
// --------------------------------------------------- Обработка СМС команд ------------------------------------------------
void MasterSms() {
  String str;
  int i, j;

  val.toLowerCase();
  if (val.indexOf(F("info")) > -1) {                               // если обнаружено кодовое слово info

    info(RingPhone, true);                                         // Информация о состоянии
  }

  if ((val.indexOf(F("clearsms")) > -1)) {                         // Удалить все СМС
    _delay_ms(1000);
    clearsms(RingPhone, true);
    _delay_ms(1000);
  } else if ((val.indexOf(F("reboot")) > -1)) {                    // Команда на перезапуск
    _delay_ms(1000);
    reboot(RingPhone);
    _delay_ms(1000);
  } else if ((val.indexOf(F("money")) > -1)) {                     // Запрос баланса
    _delay_ms(100);
    balance();
    number = String(RingPhone);
    Serial.println(RingPhone);
    _delay_ms(100);
  } else if ((val.indexOf(F("reconfig")) > -1)) {                  // Сброс конфигурации и детектирование датчиков
    reset_config();
  } else if (val.indexOf(F("window:on")) > -1) {
    window_state(true,RingPhone,false);
  } else if (val.indexOf(F("window:off")) > -1) {
    window_state(false,RingPhone,false);
  } else if (val.indexOf(F("move:on")) > -1) {
    move_state(true,RingPhone,true);
  } else if (val.indexOf(F("move:off")) > -1) {
    move_state(false,RingPhone,true);
  } else if (val.indexOf(F("door:on")) > -1) {
    open_state(true,RingPhone,true);
  } else if (val.indexOf(F("door:off")) > -1) {
    open_state(false,RingPhone,true);
  } else if ((val.indexOf(F("gas:on")) > -1)) {                    
    gas_state(true,RingPhone,true);
  } else if ((val.indexOf(F("gas:off")) > -1)) {                   // Изменение номера запроса баланса setbalnum:#105#;22
    gas_state(false,RingPhone,true);
  } else if ((val.indexOf(F("gaslevel:")) > -1)) { 
    gas_level(val,RingPhone,true);
  } else if ((val.indexOf(F("addphone:")) > -1)) {                 // Добавить/изменить телефон в памяти addphone:+79876543210ms
    add_phone(SMS);
  } else if ((val.indexOf(F("newmaster")) > -1)) {  
    EEPROM.write(EEPROM_IS_CONFIG, 0);                             // Запись нового мастер телефона по звонку
    sms(F("Make ring from new master"),String(RingPhone));
  } else if ((val.indexOf(F("balnum:")) > -1)) {                   // Изменение номера запроса баланса setbalnum:#105#;22
    set_bal_num(val, SMS);
  } else if ((val.indexOf(F("bytime[")) > -1)) {                   // Включение нагрузки на время в часах.
    by_time(val, SMS);
  } else if ((val.indexOf(F("stoptime:")) > -1)) {                   // Включение нагрузки на время в часах.
    set_stop_time(val, SMS);
  }
}
// ----------------------------------------------- Опрос датчика двери -----------------------------------------------------
void detect_open() {
  int j;
  if (OPEN_ALARM) {
    if (digitalRead(OPEN_DET) == LOW && OPEN_FLAG == false)
    {
      //  тут отправка SMS
      for (j = 0; j < CountPhone; j++){
        sms(F("ALARM! OPEN DOOR!"), String(AllowPhones[j]));
        }
      OPEN_FLAG = true; Serial.println(F("DOOR CLOSE!"));
      mGSM.println("ATD" + AllowPhones[0] + ";");
      Serial.println(F("DOOR OPEN!"));
    }
    if (digitalRead(OPEN_DET) == HIGH && OPEN_FLAG == true)
    {
      OPEN_FLAG = false; Serial.println(F("DOOR CLOSE!"));
    }
  }
}
// ----------------------------------------------- Опрос датчика окна ------------------------------------------------------
void detect_window() {
  int j;
  if (WINDOW_ALARM) {
    if (digitalRead(WINDOW_DET) == LOW && WINDOW_FLAG == false)
    { 
      _delay_ms(50);
      //  тут отправка SMS
      for (j = 0; j < CountPhone; j++){
        sms(F("ALARM! WINDOW OPEN!"), String(AllowPhones[j]));
        }
      WINDOW_FLAG = true; 
      mGSM.println("ATD" + AllowPhones[0] + ";");
      Serial.println(F("WINDOW OPEN!"));
    }
    if (digitalRead(WINDOW_DET) == HIGH && WINDOW_FLAG == true)
    {
      WINDOW_FLAG = false; Serial.println(F("WINDOW CLOSE"));
    }
  }
}
// ----------------------------------------------- Опрос датчика газа/дыма -------------------------------------------------
void detect_gas() {
  int j;
  if (GAS_ALARM) {
    if (analogRead(GAS) > GAS_ALARM_VALUE && GAS_FLAG == false)
    {
      for (j = 0; j < CountPhone; j++) {
        sms(F("ALARM! GAS!"), String(AllowPhones[j]));
        _delay_ms(2000);
      }
      GAS_FLAG = true;
      Serial.println(F("GAS!"));
    }
    }
    if (analogRead(GAS) < GAS_ALARM_VALUE && GAS_FLAG == true) GAS_FLAG = false;
}
// ---------------------------------------------------- Очистка памяти СМС -------------------------------------------------
void clearsms(String phone, boolean state) {

  mGSM.println(F("AT+CMGD=0,4"));           // Удалить СМС
  if (state)                              // Если SMS разрешено, то отправить СМС
    sms(F("Clear SMS done!"), phone);
    _delay_ms(1000);
}
// --------------------------------------------------- Отправка СМС --------------------------------------------------------
void sms(String text, String phone) { 
  
  Serial.print(F("Sending sms to: "));
  Serial.println(phone);
  mGSM.print("AT+CMGS=\"");
  mGSM.print(String(phone));                         // Кому отправка
  mGSM.println("\"");        
  _delay_ms(100);
  mGSM.print(text);                                  // Текст сообщения
  _delay_ms(100);
  mGSM.print((char)26);                              // отправка (конец сообщения)
  _delay_ms(5000);
  clearsms("", false);                               // Очистка памяти СМС
 
}
// ------------------------------------------------- Загрузка параметров ---------------------------------------------------
void load_settings() {

  EEPROM.get(EEPROM_SETTINGS_ADDRESS, dev_config);                           // Загрузка с EEPROM
  _delay_ms(100);
  Serial.println(F("Load settings..."));
  Serial.print(F("Phone_count:"));Serial.println(dev_config.phone_count);
  Serial.print(F("Phone_1:"));Serial.println(EPSR(PHONE1));
  Serial.print(F("Phone_2:"));Serial.println(EPSR(PHONE2));
  Serial.print(F("Bal_num:"));Serial.println(EPSR(BAL_NUM));
  Serial.print(F("Bal_len:"));Serial.println(dev_config.bal_len);
  Serial.print(F("Interval:"));Serial.println(dev_config.interval);
  Serial.print(F("GAS_al_val:"));Serial.println(dev_config.gas_alarm_value);
  
  CountPhone = dev_config.phone_count;                                  // Получаем количество телефонов
  OPEN_ALARM = dev_config.open_alarm;                                   // Флаг включения контроля термореле 1  
  GAS_ALARM = dev_config.gas_alarm;                                     // Флаг включения контроля датчика газа
  GAS_ALARM_VALUE = dev_config.gas_alarm_value;                         // Пороговое значение АЦП датчика газа
  WINDOW_ALARM = dev_config.window_alarm;                               // Флаг включения контроля
  MOVE_ALARM = dev_config.move_alarm;
  
  if (CountPhone == 2) {                                                // Если в памяти 2 номера
    AllowPhones[0] = EPSR(PHONE1);                                      // Получаем номера телефонов
    AllowPhones[1] = EPSR(PHONE2);
  }                                                                      
  if (CountPhone == 1)                                                   // Если в памяти 1 номер
    AllowPhones[0] = EPSR(PHONE1);                                       // Получаем номера телефонов
}
// --------------------------------------------------- Сброс конфигурации --------------------------------------------------
void reset_config() {
  float perc;
  Serial.println(F("Clear EEPROM"));
  for (int addr = 0; addr < EEPROM.length(); addr++){               // Записываем нули в EEPROM
    perc=addr/(float)EEPROM.length();
    EEPROM.write(addr, 0);
    Serial.println(String(perc*100.0));
  }
    
  Serial.println(F("Reconfiguring"));
  Serial.println(F("Write EEPROM"));

  EEPROM.write(EEPROM_IS_CONFIG, 0);                                // Пишем  параметры по умолчанию
  dev_config.phone_count = 0;
  EPSW(PHONE1, "");
  EPSW(PHONE2, "");
  EPSW(BAL_NUM, "#105#");
  dev_config.bal_len = BALANCE_LEN;
  dev_config.interval = INT_INTERVAL;
  dev_config.gas_alarm = false;
  dev_config.open_alarm = true;
  dev_config.move_alarm = false;
  dev_config.gas_alarm_value=GAS_VALUE;
  dev_config.window_alarm = false;
  dev_config.guard=false;
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  Serial.println(F("Reboot"));
  reboot(F("System reboot"));
}
// ------------------------------------------------------ Установка интервала обновления -----------------------------------
void set_interval(boolean state) {
  String msg = "";
  long interval = val.substring(val.indexOf("l:") + 2, val.length()).toInt();
  if (interval > 0 && interval < 100) {
    msg = F("Interval: ");
    msg += String(interval);
    msg += F(" secund");
    dev_config.interval = interval * 1000;
    EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
    Serial.println(msg);
    if (state) sms(msg, RingPhone);
  }
}
// -------------------------------------------------------- Установить номер запроса ---------------------------------------
void set_bal_num(String val, boolean state) {
  String msg = "";
  String buf = val.substring(val.indexOf("m:") + 2, val.indexOf(";") + 1);
  char cbuf[buf.length()];
  buf.toCharArray(cbuf, buf.length());
  EPSW(BAL_NUM, cbuf);
  dev_config.bal_len = val.substring(val.indexOf(";") + 1, val.length()).toInt();
  if (dev_config.bal_len <= 10)dev_config.bal_len = BALANCE_LEN;
  msg = F("BAL NUM:");
  msg += String(EPSR(BAL_NUM));
  msg += F("BAL LEN:");
  msg += String(dev_config.bal_len);
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  Serial.println(msg);
  if (state) sms(msg, RingPhone);
}
//-------------------------------------------------------- Изменить \ добавить номер ---------------------------------------
void add_phone(boolean state) {

  String msg = "";

  if ((val.indexOf(F("ms")) > -1)) {
    String buf = String("+") + val.substring(val.indexOf("e:") + 2, val.indexOf("ms"));
    
    char cbuf[20];
    buf.toCharArray(cbuf, 20);
    EPSW(PHONE1, cbuf);
    _delay_ms(100);
    AllowPhones[0] = String((char*)cbuf); 
    if (dev_config.phone_count==0) {
    dev_config.phone_count = 1;
    CountPhone=1;
    }
    msg = "Change master phone :" + String(AllowPhones[0]);
    Serial.println(AllowPhones[0]);
  } else {
    String buf = String("+") + val.substring(val.indexOf("e:") + 2, val.length());
    char cbuf[20];
    buf.toCharArray(cbuf, 20);
    EPSW(PHONE2, cbuf);
    dev_config.phone_count = 2;
    CountPhone=2;
    _delay_ms(100);
    AllowPhones[1] = String((char*)cbuf); 
    msg = "ADD  phone :" + String(AllowPhones[1]);
    Serial.println(AllowPhones[1]);
  }
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  msg+=F(", kol-vo - ");
  msg+=String(dev_config.phone_count);
  Serial.println(msg);
  if (state) sms(msg, RingPhone);
}
// ------------------------------------------------ Функции записи /чтения текста в /из EEPROM -----------------------------
void EPSW(int p_start_posn, char p_string[] ) { 
  //Write a string to EEPROM and terminate it with a NULL
  //We read first and do not rewrite bytes/chars that are unchanged

  for (int l_posn = 0; l_posn < (int) strlen(p_string); l_posn ++) {
    byte l_byte = (byte) p_string[l_posn];
    byte l_read = EEPROM.read(p_start_posn + l_posn);
    if (l_read != l_byte) {
      EEPROM.write(p_start_posn + l_posn, l_byte);
    }
  }
  //write the NULL termination
  if (EEPROM.read(p_start_posn + strlen(p_string)) != 0)
    EEPROM.write(p_start_posn + strlen(p_string), 0);
  //
} //EPSW
// ------------------------------------------------ Функции записи /чтения текста в /из EEPROM -----------------------------
String EPSR(int p_start_posn) { 
  //Read a NULL terminated string from EEPROM
  //Only strings up to 128 bytes are supported
  byte l_byte;

  //Count first, reserve exact string length and then extract
  int l_posn = 0;
  while (true) {
    l_byte = EEPROM.read(p_start_posn + l_posn);
    if (l_byte == 0) {
      break;
    }
    l_posn ++;
  }

  //Now extract the string
  String l_string = "";
  l_string.reserve(l_posn + 1);
  char l_char;
  l_posn = 0;
  while (true) {
    l_byte = EEPROM.read(p_start_posn + l_posn);
    if (l_byte == 0) {
      break;
    }
    l_char = (char) l_byte;
    l_string += l_char;
    l_posn ++;
    if (l_posn == 128)
      break;
    //
  }
  return l_string;
} //EPSR
// ------------------------------------ Вывод информации о текущем состоянии -----------------------------------------------
void info(String phone, boolean state) {

  String msg = F("Control v1.0 Info:");
  msg +=F("OPS:*");
  msg += Ops;
  msg +="*;";
 if (IS_RTC){
  DateTime now = rtc.now();
  msg += now.year();
  msg += F("/");
  msg += now.month();
  msg += F("/");
  msg += now.day();
  msg += F(" (");
  msg += (daysOfTheWeek[now.dayOfTheWeek()]);
  msg += F(") ");
  msg += now.hour();
  msg += F(":");
  msg += now.minute();

 }
  if (OPEN_ALARM) {msg += F(" OPEN DOOR:ON; ");} else {msg += F(" OPEN DOOR:OFF; ");} 
  if (OPEN_FLAG) {msg += F(" DOOR OPEN; ");} else {msg += F(" DOOR CLOSE; ");}
  if (GAS_ALARM) {msg += F(" GAS:ON; "); } else {msg += F(" GAS:OFF; ");} 
  if (GAS_FLAG) {msg += F(" GAS ALARM!; ");} else {msg += F(" NO GAS ALARM; ");}
  if (WINDOW_ALARM) {msg += F(" WINDOW ON; ");} else {msg += F(" WINDOW OFF; ");}
  if (WINDOW_FLAG) {msg += F(" WINDOW OPEN!; ");} else {msg += F(" WINDOW CLOSE; ");}
  for(int i=0;i<2;i++){
   if (out_state[i]) {
    msg += F(" OUT["); msg +=String(i+1); msg +=F("]:ON;");
    } else {
    msg += F(" OUT["); msg +=String(i+1); msg +=F("]:OFF;");
    }
  }
  if (digitalRead(MAIN_POWER)==LOW) {msg += F(" POWER:OFF; ");} else {msg += F(" POWER:ON; ");}
  msg += F("Battery: ");
  if (digitalRead(MAIN_POWER)==LOW) {
    voltage_i = analogRead(BATTERY);
    voltage_f = (BATTERY_VOLTAGE / 1024) * voltage_i;
    msg += String(voltage_f);
    msg += F("v;");
  } else {
    msg += F("charging; ");
    msg += String(voltage_f);
    msg += F("v;");
  }
  
  Serial.println(msg);

  if (state) {
    int cnt = msg.length() / 160;
    for (int i = 0; i < cnt; i++)
      sms(msg.substring(160 * i, 160 * i + 160), phone);
    sms(msg.substring(160 * cnt, msg.length()), phone);
  }
}
// --------------------- включение/выключение режима контроля температуры --------------------------------------------------
void open_state(boolean state, String phone, boolean sms_on) {
  int j;
  String str=F("Door:");
  OPEN_ALARM = state;
  if (state) {
    dev_config.open_alarm = true;
    str += F("ON!:");
    digitalWrite(ALARM_LED, HIGH);
  } else {
    dev_config.open_alarm = false;
    str += F("OFF!:");
    digitalWrite(ALARM_LED, LOW);
  }
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  str += phone;
  if(sms_on) {
     sms(str, phone);
    _delay_ms(2000);
   }
  Serial.println(str);
}
// --------------------- включение/выключение режима контроля температуры --------------------------------------------------
void window_state(boolean state, String phone, boolean sms_on) {
  int j;
  String str=F("Window:");
  WINDOW_ALARM = state;
  if (state) {
    dev_config.window_alarm = true;
    str += F("ON!:");
    //digitalWrite(ALARM_LED, HIGH);
  } else {
    dev_config.window_alarm = false;
    str += F("OFF!:");
    //digitalWrite(ALARM_LED, LOW);
  }
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  str += phone;
  if(sms_on) {
     sms(str, phone);
    _delay_ms(2000);
   }
  Serial.println(str);
}
// --------------------- включение/выключение режима контроля газа и дыма --------------------------------------------------
void gas_state(boolean state, String phone, boolean sms_on) {
  int j;
  String str;
  GAS_ALARM = state;
  if (state) {
    dev_config.gas_alarm = true;
    str = F("GAS:ON!:");
   } else {
    dev_config.gas_alarm = false;
    str = F("GAS:OFF!:");
  }
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  str += phone;
  if(sms_on) {
     sms(str, phone);
    _delay_ms(2000);
   }
  Serial.println(str);
}
// --------------------- включение/выключение режима контроля движения -----------------------------------------------------
void move_state(boolean state, String phone, boolean sms_on) {
  int j;
  String str=F(" MOVE:");
  MOVE_ALARM = state;
  if (state) {
    dev_config.move_alarm = true;
    str += F("ON!:");
   } else {
    dev_config.move_alarm = false;
    str += F("OFF!:");
  }
  EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
  str += phone;
  if(sms_on) {
     sms(str, phone);
    _delay_ms(2000);
   }
  Serial.println(str);
}
// ----------------------------------------- Порог датчика газа ------------------------------------------------------------
void gas_level(String val, String phone, boolean state) {
  int j,gl_val;
  String str;
   gl_val=val.substring(val.indexOf("l:") + 2, val.length()).toInt();
    if(gl_val>0 && gl_val <1024) {
         dev_config.gas_alarm_value=gl_val;
         GAS_ALARM_VALUE = dev_config.gas_alarm_value;
         EEPROM.put(EEPROM_SETTINGS_ADDRESS, dev_config);
         str = F("GAS_alarm_value:");
         str += String(gl_val);
      } else {str = F("ERROR: <0..1023>");}
    if(state) {
          sms(str, phone);    
          _delay_ms(2000);
    }

    Serial.println(str);
}
// ------------------------------------------------ Вывод телефонов --------------------------------------------------------
void get_phones(void) {

  Serial.print(F("phones:")); Serial.println(dev_config.phone_count);
  if (dev_config.phone_count == 1)                                  // Если в памяти один телефон
    Serial.print(F("ph[0]:")); Serial.println(EPSR(PHONE1));
  if (dev_config.phone_count == 2) {                                // Если в памяти два телефона
    Serial.print(F("ph[0]:")); Serial.println(EPSR(PHONE1));
    Serial.print(F("ph[1]:")); Serial.println(EPSR(PHONE2));
  }
}

void detect_temp() {
  int j;
  if (TEMP_ALARM) {

    if (now_temp > ALARM_TEMP && TEMP_FLAG == false) {
      TEMP_FLAG = true;
      
      for (j = 0; j < CountPhone; j++)
        sms(F("High temp inside!"), String(AllowPhones[j]));
    }

    if (now_temp < ALARM_TEMP && TEMP_FLAG == true)
      TEMP_FLAG = false;

  }
}
// ------------------------------------------------- Включение по времени        --------------------------------------------
void by_time(String val, boolean state) {
  String msg = "";
  int i = val.substring(val.indexOf(F("time[")) + 5,val.indexOf(F("]:"))).toInt();
  int t = val.substring(val.indexOf(F("]:")) + 2, val.length()).toInt();
     if (i>0 && i<3){
    stop_time[i-1]=millis()+3600000*t; 
  msg =F("Out:[");
  msg +=String(i);
  msg += F("] Vkluchenie na ");
  msg += String(t);
  msg += F(" h");
  Serial.println(msg);
  if (state) sms(msg, RingPhone);
 }
}
void set_stop_time(String val, boolean state) {
  int i = val.substring(val.indexOf(F("time:")) + 5,val.length()).toInt();
  
  if (i>0 && i<3){
    stop_time[i-1]=millis(); 
  } else if(i==0){
    stop_time[0]=millis();
    stop_time[1]=millis();
    }
  }
